smooth2d <- 
  function(x, y, df = 5, m = 2, REML = TRUE, control = lmerControl()){
    # tensor product smooth (2 dimensions)
    # Nathaniel E. Helwig (helwig@umn.edu)
    # 26 December 2023
    
    # check x
    x <- as.matrix(x)
    if(ncol(x) != 2L) stop("Input 'x' must be a matrix with 2 columns.")
    xnames <- c("x1", "x2")
    colnames(x) <- xnames
    
    # get nobs
    nobs <- nrow(x)
    
    # check y
    y <- as.numeric(y)
    if(length(y) != nobs) stop("Inputs 'x' and 'y' are incompatible")
    
    # check df
    df <- as.integer(df)
    if(any(df < 3L | df > nobs)) stop("Input 'df' must satisfy 3 <= df <= nobs")
    if(length(df) != 2L) df <- rep(df[1], 2)
    
    # check m
    m <- as.integer(m)
    if(any(m < 0L | m > 3L)) stop("Input 'm' must satisfy 0 <= m <= 3")
    if(length(m != 2)) m <- rep(m[1], 2)
    
    # make basis function matrix
    xlist <- vector("list", 3)
    xscale <- rep(NA, 3)
    names(xlist) <- names(xscale) <- c("x1", "x2", "x1:x2")
    for(j in 1:2){
      xlist[[j]] <- grpnet::rk(x[,j], df = df[j], m = m[j])
      xscale[j] <- sqrt(mean(xlist[[j]]^2))
      xlist[[j]] <- xlist[[j]] / xscale[j]
    }
    
    # make interaction basis
    j <- 3
    xlist[[j]] <- grpnet::row.kronecker(xlist[[1]], xlist[[2]])
    xscale[j] <- sqrt(mean(xlist[[j]]^2))
    xlist[[j]] <- xlist[[j]] / xscale[j]
    
    # create data frame (for lmer)
    data <- data.frame(y = y,
                       x1 = factor(rep(1:ncol(xlist[[1]]), length.out = nobs)),
                       x2 = factor(rep(1:ncol(xlist[[2]]), length.out = nobs)),
                       x1x2 = factor(rep(1:ncol(xlist[[3]]), length.out = nobs)))
    
    
    # make lmer formula
    lmer.form <- as.formula(y ~ (1 | x1) + (1 | x2) + (1 | x1x2))
    
    # process formula to build model matrices
    lform <- lFormula(lmer.form, data = data, control = control)
    
    # terms and labels
    myterms <- c("x1", "x2", "x1x2")
    mylabels <- names(xlist)
    
    # replace dummy coded variables
    lterm <- names(lform$reTrms$cnms)
    for(i in 1:length(lterm)){
      id <- which(myterms[i] == lterm)
      index <- (lform$reTrms$Gp[id]+1):lform$reTrms$Gp[id+1]
      lform$reTrms$Zt[index,] <- as(t(xlist[[i]]),"dgCMatrix")
      lform$reTrms$cnms[[id]] <- mylabels[i]
    }
    
    # make deviance function
    devfun <- do.call(mkLmerDevfun, lform)
    
    # optimize
    opt <- optimizeLmer(devfun, control = control$optCtrl)
    
    # package results
    res <- NULL
    res$mer <- mkMerMod(environment(devfun), opt, lform$reTrms, fr = lform$fr)
    
    # extract blups
    blups <- ranef(res$mer)
    
    # evaluate fitted values
    fit <- res$mer@beta
    coefs <- NULL
    for(k in 1:3){
      id <- which(names(blups) == myterms[k])
      coefs.k <- unlist(blups[[id]])
      coefs <- c(coefs, coefs.k)
      fit <- fit + xlist[[k]] %*% coefs.k
    }
    res$coefficients <- c(res$mer@beta, coefs)
    res$fitted.values <- fit
    res$xscale <- xscale
    res$df <- df
    res$m <- m
    return(res)
    
  } # end smooth2d