### Tensor Product Smoothing via Spectral Spline Representations: Example
### Nathaniel E. Helwig (helwig@umn.edu)
### Updated: 26 December 2023



#########***#########   LOAD PACKAGES/FUNCTIONS   #########***#########

### load packages
require(lme4)    # ver 1.1-35.1
require(grpnet)  # ver 0.2
require(lattice) # ver 0.22-5

### load tpss function
source("smooth2d.R")



#########***#########   LOAD DATA   #########***#########

### download data
download.file(url = "https://archive.ics.uci.edu/static/public/275/bike+sharing+dataset.zip",
              destfile = "bike+sharing+dataset.zip")

### unzip data
unzip("bike+sharing+dataset.zip")

### read data
bike <- read.csv("hour.csv", row.names = 1)

### get average 
avg <- with(bike, tapply(cnt, list(hr, mnth), mean))

### define x and y
x <- expand.grid(hour = 0:23, month = 1:12)
y <- c(avg)
df <- data.frame(hour = x[,1], month = x[,2], count = y)

### fit model
mod <- smooth2d(x, log10(y), df = c(12, 6))

### newx for predictions
hour <- seq(0, 23, length.out = 100)
month <- seq(1, 12, length.out = 50)
newx <- expand.grid(hour = hour, month = month)

### make basis matrices
xlist <- vector("list", 3)
xlist[[1]] <- grpnet::rk(newx[,1], df = mod$df[1], m = mod$m[1]) / mod$xscale[1]
xlist[[2]] <- grpnet::rk(newx[,2], df = mod$df[2], m = mod$m[2]) / mod$xscale[2]
xlist[[3]] <- grpnet::row.kronecker(xlist[[1]], xlist[[2]]) / mod$xscale[3]
xmat <- do.call("cbind", xlist)

### form predictions
newy <- cbind(1, xmat) %*% mod$coefficients
fit <- data.frame(hour = newx[,1], month = newx[,2], count = newy)

### plot results
pdf(file = "fig6-bike.pdf", width = 10, height = 5)
par(mar = c(4, 4, 4, 2) + 0.2)
layout(rbind(matrix(1:2, ncol = 2), 3), heights = c(0.75, 0.25))
image(0:23, 1:12, matrix(df$count, 24, 12), 
      xlab = "Hour", ylab = "Month", zlim = c(0, 600),
      cex.axis = 1.25, cex.lab = 1.5, cex.main = 2,
      main = "Averaged")
image(hour, month, matrix(10^fit$count, 100, 50), 
      xlab = "Hour", ylab = "Month", zlim = c(0, 600),
      cex.axis = 1.25, cex.lab = 1.5, cex.main = 2,
      main = "Predicted")
par(mar = c(3, 8, 3, 8))
image(seq(0, 600, length.out = 12), 1, 
      matrix(seq(0, 600, length.out = 12), ncol = 1), 
      xlab = "", ylab = "", axes = FALSE)
mtext("Number of Bike Rentals", line = 0.5, cex = 1.5 * par()$cex)
box()
axis(1, cex.axis = 1.25)
dev.off()
