### Tensor Product Smoothing via Spectral Spline Representations: Figures 1 & 2
### Nathaniel E. Helwig (helwig@umn.edu)
### Updated: 9 December 2023



### load 'grpnet' package for rk() function
library(grpnet)  # ver 0.2



#########***#########   FIGURE 1: Basis Functions   #########***#########

### open graphics device
pdf(file = "fig1-basis.pdf", width = 9, height = 6)

### graphical parameters
par(mar = c(4, 4.25, 4, 2) + 0.1, mfrow = c(2,2))
layout(cbind(matrix(1:4, 2, 2, byrow = TRUE), 5), widths = c(0.46, 0.46, 0.08))

### ordinal spline basis
x <- seq(0, 1, length.out = 101)
basis <- rk(x, m = 0, df = 5) %*% diag(c(1, 1, 1, -1, 1))
basis <- scale(basis)  # for visualization only!
plot(x, basis[,1], t = "l", ylim = extendrange(basis),
     ylab = "S(x)", main = "Ordinal Smoothing Spline", lwd = 2,
     cex.axis = 1.25, cex.lab = 1.5, cex.main = 1.75)
for(j in 2:ncol(basis)){
  lines(x, basis[,j], col = j, lwd = 2)
}
abline(h = 0, lty = 3)

### linear spline basis
x <- seq(0, 1, length.out = 101)
basis <- rk(x, m = 1) %*% diag(c(1,1,1,-1,1))
basis <- scale(basis)  # for visualization only!
plot(x, basis[,1], t = "l", ylim = extendrange(basis),
     ylab = "S(x)", main = "Linear Smoothing Spline", lwd = 2,
     cex.axis = 1.25, cex.lab = 1.5, cex.main = 1.75)
for(j in 2:ncol(basis)){
  lines(x, basis[,j], col = j, lwd = 2)
}
abline(h = 0, lty = 3)

### cubic spline basis
x <- seq(0, 1, length.out = 101)
basis <- rk(x)
basis <- scale(basis)  # for visualization only!
plot(x, basis[,1], t = "l", ylim = extendrange(basis),
     ylab = "S(x)", main = "Cubic Smoothing Spline", lwd = 2,
     cex.axis = 1.25, cex.lab = 1.5, cex.main = 1.75)
for(j in 2:ncol(basis)){
  lines(x, basis[,j], col = j, lwd = 2)
}
abline(h = 0, lty = 3)

### quintic spline basis
x <- seq(0, 1, length.out = 101)
basis <- rk(x, m = 3)
basis <- scale(basis)  # for visualization only!
plot(x, basis[,1], t = "l", ylim = extendrange(basis),
     ylab = "S(x)", main = "Quintic Smoothing Spline", lwd = 2,
     cex.axis = 1.25, cex.lab = 1.5, cex.main = 1.75)
for(j in 2:ncol(basis)){
  lines(x, basis[,j], col = j, lwd = 2)
}
abline(h = 0, lty = 3)

### legend
par(mar = rep(0.1, 4))
plot(1,1,xlab = "", ylab = "", type = "n", axes = F)
legend("right", c(expression(S[1]), 
                  expression(S[2]),
                  expression(S[3]), 
                  expression(S[4]), 
                  expression(S[5])), 
       cex = 1.5, col = 1:5, lwd = 2, bty = "n")

dev.off()





#########***#########   FIGURE 2: Tensor Product Basis   #########***#########

### make x grid
n <- 100
x <- seq(0, 1, length.out = n)
xgrid <- expand.grid(x, x)

### make marginal and tensor product basis
basis1 <- rk(xgrid[,1])
basis2 <- rk(xgrid[,2])
tpbasis <- row.kronecker(basis1, basis2)

### plot bidimensional basis functions
pdf(file = "fig2-tpbasis.pdf", width = 9, height = 9)
par(mfrow = c(5, 5), mar = c(1.25, 1.25, 1.25, 1.25) + 0.1)
for(i in 1:25){
  image(x, x, matrix(tpbasis[,i], n, n), xlab = "x1", ylab = "x2",
        main = substitute(italic(S)[ii], list(ii = i)), axes = FALSE,
        col = hcl.colors(12, "YlGnBu", rev = TRUE), cex.main = 1.5)
}
dev.off()

